<?php
/**
 * WordPress Widget Boilerplate
 *
 * The WordPress Widget Boilerplate is an organized, maintainable boilerplate
 * for building widgets using WordPress best practices.
 *
 * @package   BP_Widget_What_We_Do
 * @author    Martin Goldie <martin@brewpixel.com>
 * @license   GPL-2.0+
 * @link      http://brewpixel.com
 * @copyright 2015 BrewPixel Limited
 *
 * @wordpress-plugin
 * Plugin Name:       BrewPixel What We Do Widget
 * Plugin URI:        http://brewpixel.com
 * Description:       Widget plugin to create a widget for HubSpot calls to
 * action. Settings are available for naming among other settings.
 * Version: 1.0.1
 * Author: Martin Goldie, BrewPixel
 * Author URI: http://brewpixel.com
 * Text Domain: brewpixel-widget-what-we-do-boxes
 * License: GPL-2.0+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.txt
 * Domain Path: /lang
 */

// Prevent direct file access.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class BP_Widget_What_We_Do
 */
class BP_Widget_What_We_Do extends WP_Widget
{

	/**
	 * Unique identifier for your widget.
	 *
	 * The variable name is used as the text domain when internationalizing
	 * strings of text. Its value should match the Text Domain file header in
	 * the main widget file.
	 *
	 * @since 1.0.0
	 *
	 * @var $widget_slug    string
	 */
	protected $widget_slug = 'brewpixel-widget-what-we-do-boxes';

	/**
	 * This is the display friendly name for the widget
	 *
	 * @var mixed|void
	 */
	private $widget_name = 'What we do';

	/**
	 * This is the display friendly name for the widget area
	 *
	 * @var string
	 */
	private static $widget_area_name = 'What We Do';

	public $bp_widget_what_we_do_setting_area_name;
	/*
    * --------------------------------------------------
    * Constructor
    * --------------------------------------------------
    */

	/**
	 * Specifies the classname and description, instantiates the widget,
	 * loads localization files, and includes necessary stylesheets and
	 * JavaScript.
	 */
	public function __construct() {

		// Load plugin text domain.
		add_action( 'init', array( $this, 'widget_textdomain' ) );

		// Hooks fired when the Widget is activated and deactivated.
		register_activation_hook( __FILE__, array( $this, 'activate' ) );
		register_deactivation_hook( __FILE__, array( $this, 'deactivate' ) );
		$this->widget_name = 'What we do';
		if ( get_option( 'bp_widget_what_we_do_setting_name' ) && null !== get_option( 'bp_widget_what_we_do_setting_name' ) && get_option( 'bp_widget_what_we_do_setting_name' ) !== '' ) {
			// $this->set_widget_slug( get_option( 'bp_widget_what_we_do_setting_name' ));
			$this->widget_name = get_option( 'bp_widget_what_we_do_setting_name' );
		} else {
			update_option( 'bp_widget_what_we_do_setting_name', $this->widget_name );
		}

		// TODO: update description.
		parent::__construct(
			$this->get_widget_slug(),
			__( $this->widget_name, $this->get_widget_slug() ),
			array(
			'classname'   => $this->get_widget_slug() . '-class',
			'description' => __(
				'What we do boxes.',
				$this->get_widget_slug()
			),
			)
		);

		self::$widget_area_name = $this->get_widget_area_name();

		$this->register_widget_settings();

		// Register admin styles and scripts.
		add_action(
			'admin_print_styles',
			array( $this, 'register_admin_styles' )
		);
		add_action(
			'admin_enqueue_scripts',
			array( $this, 'register_admin_scripts' )
		);

		// Register site styles and scripts.
		add_action(
			'wp_enqueue_scripts',
			array( $this, 'register_widget_styles' )
		);
		add_action(
			'wp_enqueue_scripts',
			array( $this, 'register_widget_scripts' )
		);

		// Refreshing the widget's cached output with each new post.
		add_action( 'save_post', array( $this, 'flush_widget_cache' ) );
		add_action( 'deleted_post', array( $this, 'flush_widget_cache' ) );
		add_action( 'switch_theme', array( $this, 'flush_widget_cache' ) );
	} // end constructor


	/**
	 * Return the widget slug.
	 *
	 * @since 1.0.0
	 *
	 * @return Plugin slug variable.
	 */
	public function get_widget_slug() {

		return $this->widget_slug;
	}

	/**
	 * This is the slug for the widget.
	 *
	 * @param string $widget_slug The slug for the widget.
	 */
	public function set_widget_slug( $widget_slug ) {

		$this->widget_slug = $widget_slug;
	}

	/**
	 * If the option is set in the settings for the widget area name
	 * then use that instead of the default name.
	 *
	 * @return mixed|string|void
	 */
	public static function get_widget_area_name() {

		if ( get_option( 'bp_widget_what_we_do_setting_area_name' ) !== '' ) {
			$widgetname = self::$widget_area_name;
			// = get_option( 'bp_widget_what_we_do_setting_area_name' )
		} else {
			$widgetname = 'What We Do';
			update_option( 'bp_widget_what_we_do_setting_area_name', $widgetname );
		}

		return $widgetname;
	}

	/*
    *--------------------------------------------------
    * Widget API Functions
    *--------------------------------------------------
    */

	/**
	 * Outputs the content of the widget.
	 *
	 * @param array $args     The array of form elements.
	 * @param array $instance The current instance of the widget.
	 *
	 * @return int
	 */
	public function widget( $args, $instance ) {

		// Check if there is a cached output.
		$cache = wp_cache_get( $this->get_widget_slug(), 'widget' );

		if ( ! is_array( $cache ) ) {
			$cache = array();
		}

		if ( ! isset( $args['widget_id'] ) ) {
			$args['widget_id'] = $this->id;
		}

		if ( isset( $cache[ $args['widget_id'] ] ) ) {
			return print  wp_kses_post( $cache[ $args['widget_id'] ] );
		}

		// Go on with your widget logic, put everything into a string.
		extract( $args, EXTR_SKIP );

		$widget_string = $before_widget;

		// TODO: Here is where you manipulate your widget's values based on their input fields.
		ob_start();
		include plugin_dir_path( __FILE__ ) . 'views/widget.php';
		$widget_string .= ob_get_clean();
		$widget_string .= $after_widget;

		$cache[ $args['widget_id'] ] = $widget_string;

		wp_cache_set( $this->get_widget_slug(), $cache, 'widget' );

		print  wp_kses_post( $widget_string );
	} // end widget

	/**
	 * Flush the widget cache.
	 */
	public function flush_widget_cache() {

		wp_cache_delete( $this->get_widget_slug(), 'widget' );
	}

	/**
	 * Processes the widget's options to be saved.
	 *
	 * @param array $new_instance The new instance of values to be generated via
	 *                            the update.
	 * @param array $old_instance The previous instance of values before the
	 *                            update.
	 *
	 * @return array
	 */
	public function update( $new_instance, $old_instance ) {

		$instance = $old_instance;

		// TODO: Here is where you update your widget's old values with the new, incoming values.
		$instance['title'] = $new_instance['title'];
		$instance['content'] = $new_instance['content'];
		$instance['icon'] = $new_instance['icon'];
		$instance['link'] = $new_instance['link'];

		return $instance;
	} // end widget

	/**
	 * Generates the administration form for the widget.
	 *
	 * @param array $instance The array of keys and values for the widget.
	 */
	public function form( $instance ) {

		// TODO: Define default values for your variables.
		$instance = wp_parse_args(
			(array) $instance,
			array(
			'content' => '',
			'title' => '',
			'icon' => '',
			'link' => '',
			)
		);

		// TODO: Store the values of the widget in their own variable.
		// Display the admin form.
		include plugin_dir_path( __FILE__ ) . 'views/admin.php';
	} // end form

	/*
    * --------------------------------------------------
    * Public Functions
    * --------------------------------------------------
    */

	/**
	 * Loads the Widget's text domain for localization and translation.
	 */
	public function widget_textdomain() {

		load_plugin_textdomain(
			$this->get_widget_slug(),
			false,
			plugin_dir_path( __FILE__ ) . 'lang/'
		);
	} // end widget_textdomain

	/**
	 * Fired when the plugin is activated.
	 *
	 * @param boolean $network_wide True if WPMU superadmin uses "Network
	 *                               Activate" action, false if WPMU is
	 *                               disabled or plugin is activated on an
	 *                               individual blog.
	 */
	public function activate( $network_wide ) {

		// TODO define activation functionality here.
	} // end activate

	/**
	 * Fired when the plugin is deactivated.
	 *
	 * @param boolean $network_wide True if WPMU superadmin uses "Network
	 *                              Activate" action, false if WPMU is disabled
	 *                              or plugin is activated on an individual
	 *                              blog.
	 */
	public function deactivate( $network_wide ) {

		// TODO define deactivation functionality here
	} // end deactivate

	/**
	 * Registers and enqueues admin-specific styles.
	 *
	 * @param string $hook This is usually the admin page (e.g. post-new.php,
	 *                     post.php etc.).
	 */
	public function register_admin_styles( $hook ) {

		wp_enqueue_style(
			$this->get_widget_slug() . '-admin-styles',
			plugins_url( 'css/admin.css', __FILE__ )
		);
	} // end register_admin_styles

	/**
	 * Registers and enqueues admin-specific JavaScript.
	 *
	 * @param string $hook This is usually the admin page (e.g. post-new.php,
	 *                     post.php etc.).
	 */
	public function register_admin_scripts( $hook ) {

		if ( 'widgets.php' != $hook ) {
			return;
		}
		wp_register_script(
			$this->get_widget_slug() . '-admin-script',
			plugins_url( 'js/admin.js', __FILE__ ),
			array( 'jquery' )
		);

		$translation_array = array(
		'widget_what_we_do_protected' => get_option( 'bp_widget_what_we_do_setting' ),
		'widget_what_we_do_limit'     => get_option( 'bp_widget_what_we_do_setting_number' ),
		'widget_what_we_do_area_name' => get_option( 'bp_widget_what_we_do_setting_area_name' ),
		'widget_what_we_do_sidebar'   => 'bp_widget_what_we_do',
		);
		wp_localize_script(
			$this->get_widget_slug() . '-admin-script',
			'widget_what_we_do_info',
			$translation_array
		);
		wp_enqueue_script( $this->get_widget_slug() . '-admin-script' );
	} // end register_admin_scripts

	/**
	 * Registers and enqueues widget-specific styles.
	 */
	public function register_widget_styles() {

		wp_enqueue_style(
			$this->get_widget_slug() . '-widget-styles',
			plugins_url( 'css/widget.css', __FILE__ )
		);
	} // end register_widget_styles

	/**
	 * Registers and enqueues widget-specific scripts.
	 */
	public function register_widget_scripts() {

		wp_enqueue_script(
			$this->get_widget_slug() . '-script',
			plugins_url( 'js/widget.js', __FILE__ ),
			array( 'jquery' )
		);
	} // end register_widget_scripts

	/**
	 * Register the settings for the widget.
	 */
	public function register_widget_settings() {

		$widget_area                 = sanitize_title_with_dashes(
			self::$widget_area_name,
			'',
			'save'
		);
		$this->bp_widget_what_we_do_setting_area_name = new \BrewPixel\BrewPress\AdminSetting\Setting(
			array(
			'id'    => 'widget_what_we_do_settings_section',
			'title' => 'Widget Settings',
			// 'callback'  => 'setting_section_callback',
			'page'  => 'brew-widget-what-we-do-settings',
			'args'  => '',
			),
			array(
			'id'    => 'bp_widget_what_we_do_setting_area_name',
			'label' => 'Name of widget area',
			'callback'  => [ $this, 'setting_section_default_callback' ],
			'page'  => 'brew-widget-what-we-do-settings',
			'args'  => [
			"Name of widget area. Shortcode to use to output widget area: [$widget_area]"
			],
			),
			'text'
		);

		$bp_widget_what_we_do_setting_name = new \BrewPixel\BrewPress\AdminSetting\Setting(
			array(
			'id'    => 'widget_what_we_do_settings_section',
			'title' => 'Widget Settings',
			// 'callback'  => 'setting_section_callback',
			'page'  => 'brew-widget-what-we-do-settings',
			'args'  => '',
			),
			array(
			'id'    => 'bp_widget_what_we_do_setting_name',
			'label' => 'Name of widget',
			// 'callback'  => 'setting_section_callback',
			'page'  => 'brew-widget-what-we-do-settings',
			'args'  => [
			'Name of widget.',
			],
			),
			'text'
		);

		$bp_widget_what_we_do_setting = new \BrewPixel\BrewPress\AdminSetting\Setting(
			array(
			'id'    => 'widget_what_we_do_settings_section',
			'title' => 'Widget Settings',
			// 'callback'  => 'setting_section_callback',
			'page'  => 'brew-widget-what-we-do-settings',
			'args'  => '',
			),
			array(
			'id'    => 'bp_widget_what_we_do_setting',
			'label' => 'Protected',
			// 'callback'  => 'setting_section_callback',
			'page'  => 'brew-widget-what-we-do-settings',
			'args'  => [
			"Check if the only type of widget you want allowed in the brew widgets area to be of the $this->widget_name widget type.",
			],
			),
			'checkbox'
		);

		$bp_widget_what_we_do_setting_number = new \BrewPixel\BrewPress\AdminSetting\Setting(
			array(
			'id'    => 'widget_what_we_do_settings_section',
			'title' => 'Widget Settings',
			// 'callback'  => 'setting_section_callback',
			'page'  => 'brew-widget-what-we-do-settings',
			'args'  => '',
			),
			array(
			'id'    => 'bp_widget_what_we_do_setting_number',
			'label' => 'Number of widgets',
			// 'callback'  => 'setting_section_callback',
			'page'  => 'brew-widget-what-we-do-settings',
			'args'  => [
			'How many widgets per area? Leave blank for unlimited number.'
			],
			),
			'number'
		);

		$bp_widget_what_we_do_setting->create_plugin_page(
			array(
			'page_title' => 'BrewPixel Widget Settings',
			'menu_title' => 'What We Do Widget',
			'capability' => 'manage_options',
			'menu_slug'  => 'brew-widget-what-we-do-settings',
			)
		);

	}

	public function setting_section_default_callback() {
		/* --------------------------------------- *
* Field Callbacks
* ---------------------------------------- */
		/**
		 * This function renders the interface elements for toggling the visibility of the typekit_code element.
		 *
		 * It accepts an array of arguments and expects the first element in the array to be the description
		 * to be displayed next to the text.
		 */
		$value = ( get_option( $this->bp_widget_what_we_do_setting_area_name->settings_field['id'] ) ? get_option( $this->bp_widget_what_we_do_setting_area_name->settings_field['id'] ) : self::$widget_area_name );
		$helper_text = empty( $this->bp_widget_what_we_do_setting_area_name->settings_field['args'] ) ? '' : $this->bp_widget_what_we_do_setting_area_name->settings_field['args'][0];
		$html        = '<input type="text" id="' . $this->bp_widget_what_we_do_setting_area_name->settings_field['id'] . '" name="' . $this->bp_widget_what_we_do_setting_area_name->settings_field['id'] . '" value="' . $value . '"/><p class="description">' . $helper_text . '</p>';

		echo $html;
	}

	/**
	 * Register sidebar area and widget.
	 */
	public static function bp_what_we_do_wp_plugin_widget_init() {

		/*
        * Future @TODO register multiple areas
        *
        register_sidebar( array(
        'name' => 'What We Do Top',
        'id' => 'BP_Widget_What_We_Do_top',
        'description' => 'Widget area description',
        'before_widget' => '',
        'after_widget' => '',
        'before_title' => '',
        'after_title' => '',
        ) );
        */

		/* original $class_before_widget = 'l-cols__col p-b-l' */
		$class_before_widget = 'bp-plugin-l-cols__col';

		register_sidebar(
			array(
			'name'          => BP_Widget_What_We_Do::get_widget_area_name(),
			'id'            => 'bp_widget_what_we_do',
			'description'   => 'Primarily used for what we do. This area can have the number of widgets set and also whether to allow other widget types to be included. Find the shortcode in Settings > What We Do',
			'before_widget' => '<div class="' . $class_before_widget . '">',
			'after_widget'  => '</div>',
			'before_title'  => '',
			'after_title'   => '',
			)
		);
	}
} // end class

/**
 * If the widget area is active then display it.
 */
function bp_what_we_do_wp_plugin_widget_area() {
	if ( is_active_sidebar( 'bp_widget_what_we_do' ) ) {
		dynamic_sidebar( 'bp_widget_what_we_do' );
	};
}

/**
 * Shortcode for our widget area.
 *
 * @param array $atts Shortcode attributes.
 */
function bp_what_we_do_wp_plugin_widget_shortcode( $atts ) {
	/*
    * Maybe we can use $atts to set things like class name, how many widget areas,
    * number of widgets allowed, protected area etc.
    */
	bp_what_we_do_wp_plugin_widget_area();
}

$widget_area = sanitize_title_with_dashes(
	BP_Widget_What_We_Do::get_widget_area_name(),
	'',
	'save'
);
add_shortcode( $widget_area, 'bp_what_we_do_wp_plugin_widget_shortcode' );

/**
 * Initiate and add the widget area.
 */
function add_what_we_do_widget_area() {
	BP_Widget_What_We_Do::bp_what_we_do_wp_plugin_widget_init();
}

// Register the widget area (sidebar).
add_action( 'widgets_init', 'add_what_we_do_widget_area' );

add_action(
	'widgets_init',
	function() {
		register_widget( 'BP_Widget_What_We_Do' );
	}
);

/**
 * Make sure the BrewPress plugin is loaded.
 */
function bp_what_we_do_widget_check_plugin_requirements() {
	include_once ABSPATH . 'wp-admin/includes/plugin.php';
	if ( false === class_exists( '\BrewPixel\BrewPress\BrewPress' ) ) {
		deactivate_plugins( plugin_basename( __FILE__ ) );
		$html = '<div class="error">';
		$html .= '<p>';
		$html .= __( 'There was a problem activating the Widget Plugin.', 'brewpixel-widget-what-we-do-boxes' );
		$html .= '</p>';
		$html .= __(
			'This plugin requires that BrewPress plugin be activated.
        Please try again. Sorry about that.', 'brewpixel-bespoke'
		);
		$html .= '</div><!-- /.updated -->';
		wp_die( wp_kses_post( $html ) );
	}
}

add_action( 'plugins_loaded', 'bp_widget_check_plugin_requirements' );
